// By default the error handling in the tof library throws exceptions. This can
// be changed by defining TOF_ERROR_HANDLER. In this example we show how this
// can be done.

#include <chronoptics/tof/error.h>

#include <iostream>

// Define a class that handles the error. The error handling occurs at
// destruction, since that's when the function call has finished and the error
// pointer written into when an error has occurred.
class MessageOnError {
 public:
  ~MessageOnError() /*noexcept(false)*/ {
    if (ptr_) {
      std::cout << "Error occurred: " << tof_error_message(ptr_) << "\n";
      tof_error_delete(ptr_);
    }
  }

  /** This sets the error pointer in the c interface */
  operator tof_error_t*() { return &ptr_; }

 private:
  tof_error_t ptr_{nullptr};
};

class MessageOnError;
#define TOF_ERROR_HANDLER MessageOnError

#include <chronoptics/tof.hpp>

using namespace chronoptics;

int main(int argc, char** argv) {
  if (argc != 2) {
    std::cout << "Expected " << argv[0] << " <serial>\n";
    return -1;
  }

  // When an error occurs, the function will always return a default constructed
  // instance of what it's supposed to return

  tof::KeaCamera cam(tof::ProcessingConfig{}, argv[1]);

  // Check that we successfully constructed kea camera
  if (!cam) {
    std::cout << "Camera construction failed, any function will now fail\n";

    // Any function will fail when camera is not constructed
    cam.is_streaming();
    return -1;
  }

  // This sets an error since you have to set the streams before starting the
  // camera
  cam.start();
}
