import cv2
import numpy as np
import chronoptics.tof as tof

processing_config = tof.ProcessingConfig()
serial = "202001a"
cam = tof.KeaCamera(processing_config, serial)

cam_config = cam.getCameraConfig()
processing_config = cam_config.defaultProcessing()
cam.setProcessConfig(processing_config)

tof.selectStreams(cam, [tof.FrameType.XYZ])
cam.start()

# Wait for camera to warm up
for _ in range(60):
    frames = cam.getFrames()
    del frames

frames = cam.getFrames()
frame = frames[0]
array = np.asarray(frame)

cam.stop()

calibration = cam.getCalibration()

cam_matrix = np.array(calibration.getDepthCameraMatrix())
cam_matrix.shape = (3, 3)
cam_matrix[2, 2] = 1
print(cam_matrix)

dist_coefficients = np.array(calibration.getDepthDistortionCoefficients())
print(dist_coefficients)

rvec, _ = cv2.Rodrigues(np.identity(3))
print(rvec)

tvec = np.array([[0.0], [0.0], [0.0]])
print(tvec)

row = 200
col = 220

point = array[row, col, 0:3]
if point[2] == 0.0:
    raise RuntimeError("Can't project point with zero value")

point[0] *= -1.0
print(point)

image_point, _ = cv2.projectPoints(point, rvec, tvec, cam_matrix, dist_coefficients)
image_point = image_point[0][0]
print(image_point)

np.testing.assert_approx_equal(col, image_point[0])
np.testing.assert_approx_equal(row, image_point[1])
